name=as.vector(Sys.getenv("RINPUT"))
psname=paste(name,".ps",sep="")
inputname=paste(name,".dat",sep="")
postscript(file=psname, horizontal=TRUE, "a4")
seqlen=256

logseq=function(min, max, length){
	return (exp(seq(log(min), log(max), length=length)))
}

# Packet loss in percent
loss2ber<-function(loss,tx,bits){
	loss=loss/100
	loss=pmax(loss,0.5/tx)
	loss=pmin(loss,1-0.5/tx)
	res=1-(1-loss)^(1/bits)
	return(res)
}

# Packet loss in percent
ber2loss<-function(ber, tx, bits){
	return(100-(1-ber)^bits*100)
}

avg<-function(a){
	return (sum(a)/length(a))
}

ber2snr<-function(ber){
	return (10*log10(-qnorm(ber)))
}

snr2ber<-function(snr){
	return (pnorm(-10^(snr/10)))
}

distogram<-function(vector1, vector2, main, xlab, sub){
	min=min(vector1, vector2)
	max=max(vector1, vector2)
	breaks=seq(min, max, length=seqlen)
	h1<-hist(vector1, breaks=breaks, plot=FALSE)
	h2<-hist(vector2, breaks=breaks, plot=FALSE)
	points1=cumsum(h1$counts)
	points2=cumsum(h2$counts)
	scale1=points1[length(points1)]/100
	scale2=points1[length(points2)]/100
	points1=points1/scale1
	points2=points2/scale2
	plot(c(h1$mids, h2$mids), c(points1, points2), pch=20, main=main,
	ylab="Number of samples [%]", xlab=xlab, sub=sub)
}

reference2=function(x){
	sub="This is a reference diagram that doesn't depend on input data."
	old=par(lab=c(21,7,7))	
	ber1500=loss2ber(x,Inf, 12176)
	ber42=loss2ber(x,Inf, 512)
	ber84=loss2ber(x,Inf, 848)
	ylim=c(min(ber1500,ber42,ber84),max(ber1500,ber42,ber84))
	plot(x, ber42, main="BER vs. packetloss", type="l",
	lwd=2, xlab="Packetloss [%]", log="xy",
	ylab="Bit Error Rate (BER)", tck=1, sub=sub, ylim=ylim)
	lines(x,ber84, lwd=2)
	lines(x,ber1500, lwd=2)
	old2=par(srt=25)
	half=length(x)/2
	text(x[half],ber42[half],
	"Minimum size (42 bytes payload of ethernet frame)", pos=3)
	text(x[half],ber84[half],
	"Default ping (84 bytes payload of ethernet frame)", pos=1)
	text(x[half],ber1500[half],
	"Full size (1500 bytes payload of ethernet frame)", pos=1)
	snr1500=ber2snr(ber1500)
	snr42=ber2snr(ber42)
	snr84=ber2snr(ber84)
	ylim=c(min(snr1500,snr42,snr84),max(snr1500,snr42,snr84))
	plot(x,snr42, type="l", lwd=2, tck=1, sub=sub, ylim=ylim,
	ylab="Optical SNR [dB]", log="x", xlab="Packetloss [%]",
	main="Optical SNR vs. packetloss")
	lines(x,snr84, lwd=2)
	lines(x,snr1500, lwd=2)
	half=length(x)*3/4
	par(srt=-26)
	text(x[half],snr42[half],
	"Minimum size (42 bytes payload of ethernet frame)",
	adj=c(0.5,3))
	par(srt=-21)
	text(x[half],snr84[half],
	"Default ping (84 bytes payload of ethernet frame)",
	adj=c(0.55,-0.5))
	par(srt=-18)
	text(x[half],snr1500[half],
	"Full size (1500 bytes payload of ethernet frame)", pos=3)
	par(old2)
	par(old)
}

appendix<-function(){
	sub="This is a reference diagram that doesn't depend on input data."
	c=seq(-0.4,-18, length=seqlen)
	c=10^c
	d=10*log10(-qnorm(c))
	old=par(lab=c(19,20,7))
	plot(c,d, type="l", log="x", ylab="SNR [dB]", xlab="BER", tck=1,
	main="Signal to Noise Ratio (SNR) vs. Bit Error Rate (BER)",
	las=1, lwd=3, sub=sub)
	c=logseq(1e-8,100-1e-8,seqlen)
	reference2(c)
	par(old)
}

date<-function(date){
	date=round(date)
	result=ISOdate(1970, 1, 1, 0)+date;
}

# Loss is in %
idealdist=function(loss, tx, iloss)
# tx is number of transmitted packets
# ideal rx is ideal number of received packets
# return value is cumulative probability up to given rx
{
	return (pbinom((1-loss/100)*tx, tx, (1-iloss/100)))
}

plotloss=function(dframe, iloss5, iloss10, filename)
# rx is idealrx
# tx is number of transmitted packets
{
	distogram(dframe$loss5, dframe$loss10, paste(filename,"packetloss"),
	"Packetloss [%]", paste("Ideal loss for 5MHz is",round(iloss5, digits=2),
        "%, for 10MHz", round(iloss10, digits=2),"%."))
	min=min(dframe$loss5, dframe$loss10)
	max=max(dframe$loss5, dframe$loss10)
	ploss=seq(min, max, length=seqlen)
	res=1-idealdist(ploss, dframe$tx[1], iloss5)
	lines(ploss, res*100)
	res=1-idealdist(ploss, dframe$tx[1], iloss10)
	lines(ploss, res*100)
}

plotber=function(dframe, iloss5, iloss10, filename)
# dframe is a data frame
# idealrx is ideal rx packets from given tx packets
{
	iber5=loss2ber(iloss5, dframe$tx[1], dframe$bits[1])
	iber10=loss2ber(iloss10, dframe$tx[1], dframe$bits[1])
	distogram(dframe$ber5, dframe$ber10, paste(filename,"BER"),
	"BER", paste("Ideal BER for 5MHz is",signif(iber5, digits=3),
	", for 10Mhz is",signif(iber10, digits=3)))
	minber=min(dframe$ber5, dframe$ber10)
	maxber=max(dframe$ber5, dframe$ber10)
	ber=seq(minber, maxber, length=seqlen)
	ploss=ber2loss(ber, dframe$tx[1], dframe$bits[1])
	res=1-idealdist(ploss, dframe$tx[1], iloss5)
	lines(ber, res*100)
	res=1-idealdist(ploss, dframe$tx[1], iloss10)
	lines(ber, res*100)
}

plotsnr=function(dframe, iloss5, iloss10, filename)
# dframe is a data frame
# idealrx is ideal rx packet from given tx packets
{
	avgsnr5=avg(dframe$snr5)
	avgsnr10=avg(dframe$snr10)
	distogram(dframe$snr5, dframe$snr10, paste(filename,"SNR"),
	"SNR [dB]", paste("Average SNR for 5MHz is",round(avgsnr5,digits=2),
	"dB, for 10Mhz is",round(avgsnr10, digits=2),"dB."))
	minsnr=min(dframe$snr5, dframe$snr10)
	maxsnr=max(dframe$snr5, dframe$snr10)
        snr=seq(minsnr, maxsnr, length=seqlen)
	ber=snr2ber(snr)
	ploss=ber2loss(ber, dframe$tx[1], dframe$bits[1])
        res=idealdist(ploss, dframe$tx[1], iloss5)
        lines(snr, res*100)
        res=idealdist(ploss, dframe$tx[1], iloss10)
        lines(snr, res*100)

}

versus=function(p, main){
	attach(p)
	main2=paste(main,"packetloss")
	plot(loss5,loss10, pch=20, main=main2, xlab="Packetloss at 5MHz [%]",
	ylab="Packetloss at 10MHz [%]")
	main2=paste(main,"BER")
	plot(ber5,ber10, pch=20, main=main2, xlab="BER at 5MHz", ylab=
	"BER at 10MHz")
	main2=paste(main,"SNR")
	plot(snr5,snr10, pch=20, main=main2, xlab="SNR at 5MHz [dB]",
	ylab="SNR at 10MHz [dB]")
	detach(p)
}

process=function(p, filename){
	first=floor(p$time[1])
	p$time=(p$time-first)/60
	sub=paste("Begins at",date(first))
	p$ber10=loss2ber(p$loss10, p$tx, p$bits)
	p$ber5=loss2ber(p$loss5, p$tx, p$bits)
	p$snr10=ber2snr(p$ber10)
	p$snr5=ber2snr(p$ber5)
avgsnr10=avg(p$snr10)
avgsnr5=avg(p$snr5)
idealber10=snr2ber(avgsnr10)
idealber5=snr2ber(avgsnr5)
idealloss10=ber2loss(idealber10,p$tx[1],p$bits[1])
idealloss5=ber2loss(idealber5,p$tx[1],p$bits[1])

plot( c(p$time, p$time), c(p$loss10, p$loss5), xlab=
paste("time[min], begins at", date(first)), ylab="Packetloss [%]",
sub=paste(p$bits[1],"bits in packet, ",p$tx[1],"packets in batch,",
length(p$tx),"batches"), main=paste(filename,"packetloss"),
ylim=c(0,100), pch=20)

plot (c(p$time,p$time), c(p$ber10, p$ber5), pch=20, log="y", xlab="time [min]",
ylab="BER", main=paste(filename,"BER"), sub=sub)

plot (c(p$time, p$time), c(p$snr10, p$snr5), pch=20, xlab="time [min]", 
ylab=paste("Optical SNR [dB], average 10MHz", round(avgsnr10, digits=2),
"dB, 5MHz", round(avgsnr5, digits=2), "dB"),
main=paste(filename,"SNR"), sub=sub)

avg=avg(p$snr5-p$snr10)
plot (p$time, p$snr5-p$snr10, pch=20, xlab="time [min]", 
ylab=paste("Optical SNR difference [dB], average", round(avg, digits=2)),
main=paste(filename,"SNR difference = SNR_5MHz - SNR_10MHz")
, sub=sub)
lines(c(min(p$time),max(p$time)),c(avg,avg), lwd=4)

iloss5=avg(p$loss5)
iloss10=avg(p$loss10)
plotloss(p, iloss5, iloss10, filename)
plotber(p, iloss5, iloss10, filename)
plotsnr(p, iloss5, iloss10, filename)
versus(p, filename)

}

my_filter=function(what, core)
{
	what=as.vector(filter(what, core, method="con", sides=2))
	what=what[!is.na(what)]
	return(what)
}

process_file=function(filename)
{
	p<-read.table(filename, header=FALSE,
	col.names=c("time","loss10","loss5","tx","bits"))

	process(p, filename)
	flen=length(p$time)/16
	core=rep(1,flen)
	core=core/sum(core)
	r=list()
	r$loss5=my_filter(p$loss5, core)
	r$loss10=my_filter(p$loss10, core)
	r$time=my_filter(p$time, core)
	r$tx=round(my_filter(p$tx, core))
	r$bits=round(my_filter(p$bits, core))
	process(r,paste(filename,"filtered (filter length",
	length(core),")"))

}

process_file(inputname)
appendix()
