/*****************************************************************************
 * bertest - Program sending Ronja test pattern Ethernet frames              *
 * Copyright (C) 2005-2009 Karel 'Clock' Kulhavy                             *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation; either version 2 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to the Free Software               *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 ****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

/* Give negative count to send to infinity.
 * Fill type: 0=10MHz
 *            1= 5MHz
 *            2=random content
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/stat.h>

#include <netinet/in.h>
#include <netpacket/packet.h>
#include <net/ethernet.h>
#include <net/if.h>
#include <fcntl.h>

#define ERR {perror(progname); exit(-1); }
#define ERRFNAME {fprintf(stderr,"%s: %s: ",progname, filename); \
	perror(NULL); exit(-1); }

#define HDRLEN 14
#define MAXPAYLOAD 1500

int ifindex;
int skt;
int payload=MAXPAYLOAD;
int filltype=0; /* 0=10MHz, 1=5MHz, 2=random content */

unsigned char mac[8];
unsigned char *progname;
unsigned char frame[MAXPAYLOAD+HDRLEN];
unsigned char *filename;
unsigned char *ifname="eth0";

struct sockaddr_ll addr;

/* s is socket */
void getinfo(unsigned char *name)
{
	struct ifreq ifr;
	
	strncpy(ifr.ifr_name,name,IFNAMSIZ);
	filename=name;
	if (0>ioctl(skt,SIOCGIFINDEX,&ifr)) ERRFNAME
	ifindex=ifr.ifr_ifindex;
	if (0>ioctl(skt,SIOCGIFHWADDR)) ERR
	memcpy(mac,&(ifr.ifr_hwaddr.sa_data),8);
	addr.sll_family=AF_PACKET;
	addr.sll_protocol=0x4254; /* "TB" in little endian */
	addr.sll_ifindex=ifindex;
	memcpy(addr.sll_addr, mac, 6);
	addr.sll_halen=6;
}

void sendframe(void)
{
	memcpy(frame,addr.sll_addr,6);
	memcpy(frame+6,addr.sll_addr,6);
	frame[12]='T';
	frame[13]='B';
retry:
	if (0>sendto(skt, frame, payload+HDRLEN, 0, (struct sockaddr *)&addr,
			sizeof(addr))){
		if (errno==ENOBUFS) goto retry;
		ERR
	}
}

/* Returns number of bytes read */
int my_read(int fd, void *buf, int len)
{
	int retval;
	int readbytes=0;

	while(1){
		retval=read(fd,buf,len);
		if (retval>0){
			readbytes+=retval;
			len-=retval;
			buf+=retval;
		}else if (!retval){
			return readbytes;
		}else switch(errno){
			case EINTR:
			case EAGAIN:
				break;
			default: ERRFNAME
		}
	}
	
}

void fill (void)
{
	int u;

	switch(filltype){
		case 1:
			/* 5MHz */
			memset(frame+HDRLEN,0x55, payload);
			break;
		case 2:
			/* Random content */
			filename="/dev/urandom";
			if (0>(u=open(filename,O_RDONLY))) ERRFNAME
			if (payload!=my_read(u,frame+HDRLEN,payload)){
				fprintf(stderr,
				"%s: short read from %s\n",
				progname, filename);
				exit(1);
			}
			close(u);
			break;
		case 0:
		default:
			/* 10 MHz */
			memset(frame+HDRLEN, 0, payload);
			break;
		}
			
}

int main(int argc, char **argv)
{
	int count=1;

	progname=argv[0];
	if (argc<1) ERR
	if (argc>1) count=atol(argv[1]);
	if (argc>2) ifname=argv[2];
	if (argc>3) payload=atol(argv[3]);
	if (argc>4) filltype=atol(argv[4]);
	if (payload>MAXPAYLOAD||payload<42){
		fprintf(stderr,"%s: invalid payload length: allowed range is"
				" 42-%d inclusive.\n", progname, MAXPAYLOAD);
		exit(-1);
	}
	fill();
	if (0>(skt=socket(PF_PACKET, SOCK_RAW, htons(0x4254)))) ERR
	/* fprintf(stderr,"s=%d\n",s); */
	getinfo(ifname);
	for (;count; ){
		sendframe();
		if (count>=0) count--;
	}
	return 0;
}
